@@ -33,7 +33,7 @@ times =[]
 time_labels =[]
 
 def version():
-    print('Rosahami processor, 8.4.2022')
+    print('Rosahami processor, 1.0.5  (31.8.2022)')
     print('Rossendorfer SAXS HAPG mirror.')
     print('Michal Smid, HZDR')
 
@@ -105,6 +105,60 @@ def run_simulation(p):
     return spectrorrr
 
 
+def run_SAXS_simulation(p):
+    p['source']['continuumAdd']=True
+    p['source']['continuumMarks']=False
+    p['source']['continuumAddedRatio']=0.
+    p['source']['divergenceAutomatic']=True
+    p['simulation']['collectLostBeams']=False
+
+    if p['simulation']['numraysE']>0:
+        p['simulation']['numraysB']=round(10**(p['simulation']['numraysE']))
+        p['simulation']['numraysM']=round(10**(p['simulation']['numraysE']-1))
+
+
+    p=mmpxrt.geometry(p)
+    sim=p['simulation']
+    s=p['source']
+    sg=p['sg']
+    nump=sim['num_processes'];
+
+    s['rOffset']=[0,0,0]
+    sg['windowX']=10
+    sg['windowY']=10
+    start = time.time()
+    s['showrealspatial']=False
+    s['continuum']=False
+    s['rOffsetRatio']=0.
+    s['rOffset']=[0,0,0]
+    divRing=s['divergenceRing']
+
+    ## SAXS run
+    print( '\nGoing to raytrace (ring)')
+    s['divergenceRing']=divRing
+    s['divergenceRectangular']=False;
+    sg['numrays']=p['simulation']['numraysR']
+    sg['numraysR']=p['simulation']['numraysR']
+    if nump>1:
+        sg['numrays']=int(np.round(sg['numraysR']/nump))
+        ringrrr = mmpxrt.raytrace_multiprocess(p,nump)
+    else:
+        ringrrr = mmpxrt.raytrace(p,None,None)
+
+    print(' ')
+    end = time.time()
+
+    spectrorrr={}
+    spectrorrr['elapsedTime']=end-start
+    spectrorrr['ring']=ringrrr
+
+    fn=p['simulation']['out_data_directory']  +'mmpxrt_results_' +p['simulation']['name']
+    pickle.dump( spectrorrr, open( fn, "wb" ) )
+    fn=p['simulation']['out_data_directory']  +'mmpxrt_parameters_' +p['simulation']['name']
+    pickle.dump( p, open( fn, "wb" ) )
+
+    return spectrorrr    
+    
 # %%##############################################################################
 def evaluate_simulation(p,spectrorrr,quarterring,climmax):
     so=p['source']
@@ -285,6 +339,10 @@ def drawDet(p,rrr,ax,pxsize,climmax):
     eds=[edX,edY]
     pos=posAll
     nR = np.histogram2d(pos[:,0],pos[:,1],eds)[0]
+    if np.size(nR)==0:
+        print("There is nothing reaching the detector, I'm failing.")
+        return
+        
     nR[0,0]=0
     nR[np.shape(nR)[0]-1,np.shape(nR)[1]-1]=0
     nR[0,np.shape(nR)[1]-1]=0
@@ -475,7 +533,7 @@ def make_unwrap_map(rrr,p,map_width,map_height,map_resolution):
 
 # %%##############################################################################
 #def unwrap_map_pictures(cnts,alYmap,valid,detY,yp,xp,alZmap,zp,detD,wD,wY,name):
-def unwrap_map_pictures(t,t2):
+def unwrap_map_pictures(t,t2,E0):
     ## % MAKING THE MODEL::: subtracting the linear thingy
     alYmap=t['alYmap']
     alZmap=t['alZmap']
@@ -487,7 +545,7 @@ def unwrap_map_pictures(t,t2):
     wD=t2['wD']
     wY=t2['wY']
     name=t2['name']
-    
+
     valid=cnts>5
 
     alYmap[np.invert(valid)]=np.nan
@@ -495,22 +553,28 @@ def unwrap_map_pictures(t,t2):
     xp=np.arange(np.size(yp))
     sel=np.isfinite(yp)
     p=np.polyfit(xp[sel],yp[sel],1)
-    pmrad=np.polyfit(xp[sel],yp[sel]*1e3,1)
     ypf=np.polyval(p,xp)
     on=np.ones(np.shape(alYmap)[1])
     alYmodel=np.matmul(np.transpose(np.matrix(ypf)),np.matrix(on))
 
-    pr=np.polyfit(detY[sel],yp[sel]*1e3,1)  #fit in real units
-    print(pr)
+    prY=np.polyfit(detY[sel],yp[sel]*1e3,1)  #fit in real units
 
     alZmap[np.invert(valid)]=np.nan
     zp=np.nanmean(alZmap,0)
     xp=np.arange(np.size(zp))
     sel=np.isfinite(zp)
-    p=np.polyfit(xp[sel],zp[sel],2)  #fitting: quadratic
+    quadratic=0
 
-    pr=np.polyfit(detD[sel],zp[sel]*1e3,2)  #fint in real units
-    print(pr)
+    lambd=12398/E0/10 #nm
+    qc=2*np.pi/lambd #conversion between divergence angle alpha=2theta[mrad] and q [nm-1]
+    print(qc)
+
+    if quadratic:#fitting: quadratic
+        p=np.polyfit(xp[sel],zp[sel],2)
+        prX=np.polyfit(detD[sel],zp[sel]*1e3,2)  #fint in real units
+    else:
+        p=np.polyfit(xp[sel],zp[sel],1)
+        prX=np.polyfit(detD[sel],zp[sel]*1e3,1)  #fint in real units
 
     zpf=np.polyval(p,xp)
 
@@ -523,17 +587,26 @@ def unwrap_map_pictures(t,t2):
     plt.imshow(alYmap*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
     prof=alYmap[:,int(np.shape(alYmap)[1]/2)]
     plt.plot(prof*500,detY,'w')
-    plt.title('alY [mrad]')
+    plt.title('2Theta Y [mrad]  (E_xfel= {:.0f} eV'.format(E0))
     plt.xlabel('detector X [mm]')
 
     #plt.xlabel('detector X [mm]')
     plt.ylabel('detector Y [mm]')
     plt.plot(ypf*500,detY,'r',linewidth=1)
-
+    plt.text(-10,2,'2Th_Y ~ {:.3f} y '.format(prY[0]),fontsize=14,color='w')
+    plt.text(-10,5,'q_y [nm-1] ~= {:.4f} y [mm]'.format(prY[0]/qc),fontsize=14,color='w')
 
 
     plt.axes(axs[1])#################################################################
-    plt.title('alZ [mrad]')
+    plt.title('2Theta X [mrad]')
+    if quadratic:
+        plt.text(-10,2,'2Th_X ~ {:.3f} x + {:.3f} x^2'.format(prX[1],prX[0]),fontsize=14,color='w')
+        plt.text(-10,5,'aspect change ~ {:.3f} '.format(prY[0]/prX[1]),fontsize=14,color='w')
+    else:
+        plt.text(-10,2,'2Th_X ~ {:.3f} x'.format(prX[0]),fontsize=14,color='w')
+        plt.text(-10,5,'aspect change ~ {:.3f} '.format(prY[0]/prX[0]),fontsize=14,color='w')
+        plt.text(-10,7,'q_x [nm-1] ~= {:.4f} x [mm]'.format(prX[0]/qc),fontsize=14,color='w')
+
 
     plt.imshow(alZmap*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
     prof=alZmap[int(np.shape(alZmap)[0]/2),:]
@@ -551,11 +624,14 @@ def unwrap_map_pictures(t,t2):
 
     plt.axes(axs[2])
     alDev=np.power( (np.power(alZmap-alZmodel,2)+ np.power(alYmap-alYmodel,2)) , 0.5)
-    alAmap=np.power( (np.power(alYmap,2)+ np.power(alZmap,2)) , 0.5)
+ #   alAmap=np.power( (np.power(alYmap,2)+ np.power(alZmap,2)) , 0.5)
 
     plt.imshow(alDev*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
     plt.title('deviation [mrad]')
-    plt.clim(0,0.2)
+    if quadratic:
+        plt.clim(0,0.2)
+    else:
+        plt.clim(0,0.7)
     plt.colorbar()
     plt.xlabel('detector X [mm]')
 
@@ -563,29 +639,29 @@ def unwrap_map_pictures(t,t2):
     plt.savefig('mmpxrt_unwrapmap_'+name+'_quadratic.png' , bbox_inches ='tight',dpi=140)
 
 
-    # %% paper picture
-    fig, axs  = plt.subplots(1, 1, sharey='row',figsize=(6,5))
+    if 0:# %% paper picture
+        fig, axs  = plt.subplots(1, 1, sharey='row',figsize=(6,5))
 
-    plt.title(r'2$\theta$[mrad]')
+        plt.title(r'2$\theta$[mrad]')
 
-    plt.imshow(alZmap*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
-    prof=alZmap[int(np.shape(alZmap)[0]/2),:]
+        plt.imshow(alZmap*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
+        prof=alZmap[int(np.shape(alZmap)[0]/2),:]
 
 
-    alDev=np.power( (np.power(alZmap-alZmodel,2)+ np.power(alYmap-alYmodel,2)) , 0.5)
-#    alAmap=np.power( (np.power(alYmap,2)+ np.power(alZmap,2)) , 0.5)
+        alDev=np.power( (np.power(alZmap-alZmodel,2)+ np.power(alYmap-alYmodel,2)) , 0.5)
+    #    alAmap=np.power( (np.power(alYmap,2)+ np.power(alZmap,2)) , 0.5)
 
-    plt.imshow(alDev*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
-    plt.title('absolute deviation [mrad]')
-    plt.xlabel('detector x [mm]')
-    plt.ylabel('detector y [mm]')
-    plt.clim(0,0.2)
-    plt.colorbar()
+        plt.imshow(alDev*1e3,extent=(-wD,wD,wY,0),cmap=rofl.cmap())
+        plt.title('absolute deviation [mrad]')
+        plt.xlabel('detector x [mm]')
+        plt.ylabel('detector y [mm]')
+
+        plt.colorbar()
 
 
 
 
-    plt.savefig('mmpxrt_unwrapmap_'+name+'_paperc.png' , bbox_inches ='tight',dpi=300)
+        plt.savefig('mmpxrt_unwrapmap_'+name+'_paperc.png' , bbox_inches ='tight',dpi=300)
 
 
 # %%##############################################################################
@@ -661,7 +737,7 @@ def make_radial_scattering(alRange,sigdown,sigup=[],points=[]):
     r=points[3,:]
     if sigup==[]:
         ff=np.interp(r,alRange,sigdown)
-    else:        
+    else:
 #        upsel=points[1,:]>0
         downsel=points[1,:]>0
         ffup=np.interp(r,alRange,sigup)
@@ -711,7 +787,7 @@ def do_unwrap_old(im,p,interpAlX,interpAlY):
                 continue
             if np.abs(y)>halfheight_mm:
                 continue
-            
+
             #correction for crystal offset
             x2=x
             if p['horizontal_crystal_offset']!=0:
@@ -753,7 +829,7 @@ def do_unwrap(im,p,interpAlX,interpAlY):
 
     xs=(np.arange(np.shape(im)[1])-p['rough_center_x'])*p['pxsize']  #rough coordinates of all pixels in JF[ mm]
     ys=(np.arange(np.shape(im)[0])-p['rough_center_y'])*p['pxsize']
-    
+
 
     i=0
     #mapping: going through each detected pixel, assigning an alpha angles to it
@@ -766,12 +842,16 @@ def do_unwrap(im,p,interpAlX,interpAlY):
     xs2=(xs-xoff)*xstretch/100  #finer coordinates of all JF pixels [mm]
     ys2=np.abs(ys-yoff)*ystretch/100
     ys2=(ys-yoff)*ystretch/100
-        
+
     ysm,xsm=np.meshgrid(ys2,xs2)
     sel1=np.abs(xsm)<p['halfwidth_mm']
     sel2=np.abs(ysm)<p['halfheight_mm']
     sel=np.logical_and(sel1,sel2)
 
+    if p['horizontal_crystal_offset']!=0:
+        sel_hco=ysm<0
+        xsm[sel_hco]=xsm[sel_hco]+p['horizontal_crystal_offset']
+
     aXs=interpAlX(xsm,ysm,grid=False)
     aYs=interpAlY(xsm,ysm*-1,grid=False)
     aYs_m=interpAlY(xsm,ysm,grid=False)
@@ -844,8 +924,8 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawcli
         plt.axis('equal')
         plt.xlim(-mradmax,mradmax)
         plt.ylim(-mradmax,mradmax)
-        it='xoff {:.0f}\nyoff {:.0f} \nXstretch {:.1f}\nYstretch {:.1f}'.format(p['xoff_um'],p['yoff_um'],p['xstretch'],p['ystretch'])
-        it=it+"\nhor.cryst.off{:.1f}".format(p['horizontal_crystal_offset'])        
+        it='xoff {:.0f}\nyoff {:.0f} \nXstretch {:.1f}\nYstretch {:.1f}\nrotation {:.0f}°'.format(p['xoff_um'],p['yoff_um'],p['xstretch'],p['ystretch'],p['rotation'])
+        it=it+"\nhor.cryst.off{:.1f}".format(p['horizontal_crystal_offset'])
         plt.text(-mradmax+2,mradmax-6,it,color='red')
 
     # %p
@@ -889,10 +969,10 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawcli
             selleft=points[0,:]<0
             plt.scatter(points[0,selleft]*1e3,points[1,selleft]*1e3,c=np.log10(points[6,selleft]),s=marksize)
             plt.clim(cl1,cl2)
-    
+
             selleft=points[0,:]>0
             plt.scatter(points[0,selleft]*1e3,points[1,selleft]*1e3,c=np.log10(points[7,selleft]),s=marksize)
-    
+
             plt.colorbar()
             plt.axis('equal')
             plt.xlim(-mradmax,mradmax)
@@ -958,23 +1038,32 @@ def draw_unwrapping(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawcli
         plt.savefig('unwrapped_'+p['name']+pars+' .png' , bbox_inches ='tight',dpi=140)
 
 
-def draw_unwrapping_fast(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawclimmax=20,radialclimmax=1,radialplotymax=20,radialplotymin=1):
+def draw_unwrapping_fast(p,points,im,alRange,sigup,sigdown,sigupndF,sigdownndF,rawclimmax=20,radialclimmax=1,radialplotymax=20,radialplotymin=1,second=1):
     mradmax=p['draw_max_mrad']
     marksize=3
 
     if 1:
         plt.figure(figsize=(13,10))
 
-        it='xoff {:.0f}\nyoff {:.0f} \nXstretch {:.1f}\nYstretch {:.1f}\nrotation {:.0}°'.format(p['xoff_um'],p['yoff_um'],p['xstretch'],p['ystretch'],p['rotation'])
-        it=it+"\nhor.cryst.off{:.1f}".format(p['horizontal_crystal_offset'])        
+        it='xoff {:.0f}\nyoff {:.0f} \nXstretch {:.1f}\nYstretch {:.1f}\nrotation {:.0f}°'.format(p['xoff_um'],p['yoff_um'],p['xstretch'],p['ystretch'],p['rotation'])
+        it=it+"\nhor.cryst.off{:.1f}".format(p['horizontal_crystal_offset'])
         plt.text(-mradmax+2,mradmax-6,it,color='red',fontsize=20)
 
-        plt.title('flatfield')
- #       xi,yi,imi,ff,d=interpolate_scattering(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_flatfield=1)
-  #      plt.pcolor(xi,yi,ff)
-        plt.scatter(points[0,:]*1e3,points[1,:]*1e3,c=points[4,:],s=marksize)
-        #plt.clim(2000,3000)#
-        plt.clim(0.5,2.0)
+        if not second:
+            plt.title('flatfield')
+    #       xi,yi,imi,ff,d=interpolate_scattering(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_flatfield=1)
+    #      plt.pcolor(xi,yi,ff)
+            plt.scatter(points[0,:]*1e3,points[1,:]*1e3,c=points[4,:],s=marksize)
+            #plt.clim(2000,3000)#
+            plt.clim(0.5,2.0)
+        else:
+            plt.title('2nd flatfield')
+     #       xi,yi,imi,ff,d=interpolate_scattering(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_flatfield=1)
+      #      plt.pcolor(xi,yi,ff)
+            plt.scatter(points[0,:]*1e3,points[1,:]*1e3,c=points[8,:],s=marksize)
+            #plt.clim(2000,3000)#
+            plt.clim(0.5,2.0)
+
         plt.axis('equal')
         plt.xlim(-mradmax,mradmax)
         plt.ylim(-mradmax,mradmax)
@@ -1318,7 +1407,7 @@ def do_flatfield(points,p,separated=0):
     da=p['alpha_step']
     alRange=np.arange(0,p['alpha_max'],da)
     sel_crystal=points[2,:]>p['lower_threshold']
-    
+
     sigup = do_radial_average(alRange,points,(points[1,:]<0)*sel_crystal)
     sigdown = do_radial_average(alRange,points,(points[1,:]>0)*sel_crystal)
 
@@ -1403,14 +1492,14 @@ def interpolate_scattering(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_fla
         flatfielded = data/ff
         return xi,yi,data,ff,flatfielded
     else:
-        return xi,yi,data,data*0,data*0      
-    
+        return xi,yi,data,data*0,data*0
+
 def interpolate_scattering_q(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_flatfield=1,XFEL_photon_energy=8150):
     nx=points[0,:]#rad
     ny=points[1,:]
-    
+
     lambd=12398/XFEL_photon_energy/10 #nm
-    qc=2*np.pi/lambd #conversion between divergence angle alpha=2theta[mrad] and q [nm-1]                
+    qc=2*np.pi/lambd #conversion between divergence angle alpha=2theta[mrad] and q [nm-1]
     qx=qc*nx
     qy=qc*ny
 
@@ -1424,25 +1513,54 @@ def interpolate_scattering_q(points,x_max=12,x_step=0.1,y_max=12,y_step=0.1,do_f
         flatfielded = data/ff
         return xi,yi,data,ff,flatfielded
     else:
-        return xi,yi,data,data*0,data*0      
+        return xi,yi,data,data*0,data*0
 
 
+def radial_average_q(points,r_max=12,r_step=0.1,XFEL_photon_energy=8150):
+#    nx=points[0,:]#rad
+ #   ny=points[1,:]
+
+    lambd=12398/XFEL_photon_energy/10 #nm
+    qc=2*np.pi/lambd #conversion between divergence angle alpha=2theta[mrad] and q [nm-1]
+ #   qx=qc*nx
+  #  qy=qc*ny
+
+#    xi = np.arange(-x_max,x_max,x_step)
+ #   yi = np.arange(-y_max,y_max,y_step)
+  #  xi,yi = np.meshgrid(xi,yi)
+
+    ri = np.arange(0,r_max,r_step)
+    alRange=ri/qc
+    sel=points[0,:]*0+1
+    data=do_radial_average(alRange,points,sel,column = 2)
+    ff=do_radial_average(alRange,points,sel,column = 5)
+
+#    data = griddata((qx,qy),points[2,:],(xi,yi),method='linear')
+#    ff = griddata((qx,qy),points[5,:],(xi,yi),method='linear')
+    flatfielded = data/ff
+#        return xi,yi,data,ff,flatfielded
+ #   else:
+  #      return xi,yi,data,data*0,data*0
+    
+    return ri,data,ff,flatfielded
+
 def tick(lab=''):
     global times, time_labels
-    import time 
+    import time
     times.append(time.time())
     time_labels.append(lab)
-    
+
 def print_times():
+    tick('end')
     global times, time_labels
     for i,t in enumerate (np.diff(times)):
         lab=time_labels[i]
         if lab!='':
             print("{:.0f}: {:.1f}s (".format(i+1,t)+lab+')')
-        else:                
-            print("{:.0f}: {:.1f}s".format(i+1,t))    
-        
+        else:
+            print("{:.0f}: {:.1f}s".format(i+1,t))
+
 def clear_times():
     global times, time_labels
     times=[]
-    time_labels=[]        
\ No newline at end of file
+    time_labels=[]
